local AddonName = "Dominos"
local Addon = LibStub('AceAddon-3.0'):GetAddon(AddonName)

local Overlay = Addon:CreateClass('Frame')
Addon.Overlay = Overlay

local Lib = LibStub('LibSharedMedia-3.0', true)

function Overlay:New(parent)
	local name = ('%s_%s_OverlayHandler'):format(AddonName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local frame = self:Bind(CreateFrame('Frame', ('%s_%s_OverlayHandler'):format(AddonName, parent.id), parent.box))
	frame.texture = frame:CreateTexture(nil, "ARTWORK")
	frame.texture:SetAllPoints(frame)
	frame.owner = parent

	frame.glow = frame:CreateTexture(nil, "ARTWORK")
	frame.glow:SetPoint("TopLeft", 35, -8)
	frame.glow:SetSize(190, 66)

	frame.glow:SetTexture("Interface\\CharacterFrame\\UI-Player-Status")
	frame.glow:SetTexCoord(0, 0.74609375, 0, 0.53125)
	frame.glow:SetBlendMode("ADD")
--	frame.glow:Hide()
	parent.threatIndicator = frame.glow
	parent.unit = parent.id
	frame.glow.unit = parent.id
	
	frame.statusCounter = 0
	frame.statusSign = -1
		
	return frame
end

function Overlay:Load()
	self:SetAllPoints(self.owner.box)
	self:SetFrameLevel(30)
	self:Show()
end

Overlay.defaults = {
	enable = true,
}

function Overlay:Layout()
	local classification = UnitClassification(self.owner.id);
	local texture
	if ( classification == "minus" ) then
		texture = ("Interface\\TargetingFrame\\UI-TargetingFrame-Minus");
		self.power:Hide();
	elseif ( classification == "worldboss" or classification == "elite" ) then
		texture = ("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");
	elseif ( classification == "rareelite" ) then
		texture = ("Interface\\TargetingFrame\\UI-TargetingFrame-Rare-Elite");
	elseif ( classification == "rare" ) then
		texture = ("Interface\\TargetingFrame\\UI-TargetingFrame-Rare");
	else
		texture = ("Interface\\TargetingFrame\\UI-TargetingFrame");
	end
	
	self.texture:SetTexCoord(0.09375, 1.0, 0, 0.78125)
	self.texture:SetTexture(texture)
	if self.owner.id == "player" then
		self.texture:SetTexCoord(1, 0.09375, 0, 0.78125)
	end
	if self.sets.enable == true then
		self:SetAlpha(1)
	else
		self:SetAlpha(0)
	end
end

function Overlay:Update(elapsed)
    if ( UnitHasVehiclePlayerFrameUI(self.owner.id) ) then
        self.glow:Hide()
    elseif (self.owner.id == "player") and( IsResting() ) then
        self.glow:SetVertexColor(1.0, 0.88, 0.25, 1.0);
        self.glow:Show();

    else
	self.glow:SetVertexColor(1,1,1,1);
        self.glow:Hide();
    end
 
     if ( self.glow:IsShown() ) then
        local alpha = 255;
        local counter = self.statusCounter + elapsed;
        local sign    = self.statusSign;
 
        if ( counter > 0.5 ) then
            sign = -sign;
            self.statusSign = sign;
        end
        counter = mod(counter, 0.5);
        self.statusCounter = counter;
 
        if ( sign == 1 ) then
            alpha = (55  + (counter * 400)) / 255;
        else
            alpha = (255 - (counter * 400)) / 255;
        end
        self.glow:SetAlpha(alpha);
    end
end

local options = {
	{
		name = "Enable",
		kind = "CheckButton",
		key = "enable",
	},
}
local opts = Dominos.Options

function Overlay:GetPanel(menu)
	if not opts.create then
		return
	end
	Overlay.panel = Overlay.panel or opts.create.Panel(menu, "Overlay", options)
	
	return Overlay.panel
end

local unitBar = Dominos[...]

unitBar:AddPart("overlay", function(parent)
	return Overlay:New(parent)
end)